/***************************************************************/
/* Dustin Graves                                               */
/* CSCI 185 10                                                 */
/* 11/25/98                                                    */
/***************************************************************/

class Diffuse
{

  //Instance variables
  protected double fIdiff[] = new double[3];         //Ambient Intensity - I = IaKa 
  protected double fKd[] = new double[3];         //Coefficient of ambient reflection at intersection
  protected double fId[] = new double[3];
  protected Vector3d fPosition = new Vector3d (0.0, 0.0, 0.0);

  //Constructors
  public Diffuse ()
  {
    int i;

    //Set defaults
    for (i = 0; i < 3; i++)
    {
      fId[i] = 1.0;
      fKd[i] = 0.0;
      fIdiff[i] = 0.0;
    }
  } //Diffuse
  
  public void setIntensity (double r, double g, double b)
  {
    fId[0] = r;
    fId[1] = g;
    fId[2] = b;
  }

  public void setIntensity (double rgb[])
  {
    fId[0] = rgb[0];
    fId[1] = rgb[1];
    fId[2] = rgb[2];
  }

  public void setCoefficient (double r, double g, double b)
  {
    fKd[0] = r;
    fKd[1] = g;
    fKd[2] = b;
  }

  public void setCoefficient (double rgb[])
  {
    fKd[0] = rgb[0];
    fKd[1] = rgb[1];
    fKd[2] = rgb[2];
  }

  public void setLightPosition (Vector3d light)
  {
    fPosition.fX = light.fX;
    fPosition.fY = light.fY;
    fPosition.fZ = light.fZ;
  }

  //Given the point of intersection and the surface normal for that point
  //Calculate the diffuse intensity
  public void computeIntensity (Vector3d intersect, Vector3d normal)
  {
    double dot;
    Vector3d light = new Vector3d (fPosition.fX - intersect.fX, fPosition.fY - intersect.fY, fPosition.fZ - intersect.fZ);

    light = light.normalize ();
    dot = normal.dotProduct (light);

    if (dot < 0)
      fIdiff[0] = fIdiff[1] = fIdiff[2] = 0.0;
    else
    {
      fIdiff[0] = fId[0] * fKd[0] * dot;
      fIdiff[1] = fId[1] * fKd[1] * dot;
      fIdiff[2] = fId[2] * fKd[2] * dot;
    }
  }

  //Fills a given array with intensity.  Returns pointer to array for convenience.
  public double[] getRGB (double out[])
  {
    out[0] = fIdiff[0];
    out[1] = fIdiff[1];
    out[2] = fIdiff[2];

    return out;
  }
  
  public float getRed ()
  {
    return (float) fIdiff[0];
  }

  public float getGreen ()
  {
    return (float) fIdiff[1];
  }

  public float getBlue ()
  {
    return (float) fIdiff[2];
  }
} //Diffuse
